"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function xmlQuery(node) {
    var wrappedNode = {
        __xmlQuery: true,
        originalNode: node,
        attributes: {},
        elements: [],
    };
    if (node == null || typeof node === 'string' || typeof node === 'number') {
        return wrappedNode;
    }
    else if ('__xmlQuery' in node) {
        return node;
    }
    else if (Array.isArray(node)) {
        var elements = node.flatMap(function (nd) { return nd.elements || []; });
        wrappedNode.elements = elements;
        wrappedNode.query = query.bind(wrappedNode, elements);
        wrappedNode.queryAll = queryAll.bind(wrappedNode, elements);
        wrappedNode.forEach = forEach.bind(wrappedNode, node);
        wrappedNode.map = map.bind(wrappedNode, node);
        wrappedNode.innerText = innerText.bind(wrappedNode, elements);
    }
    else {
        Object.assign(wrappedNode, node);
        var elements = node.elements || [];
        wrappedNode.query = query.bind(wrappedNode, elements);
        wrappedNode.queryAll = queryAll.bind(wrappedNode, elements);
        wrappedNode.forEach = forEach.bind(wrappedNode, elements);
        wrappedNode.map = map.bind(wrappedNode, elements);
        wrappedNode.innerText = innerText.bind(wrappedNode, elements);
    }
    return wrappedNode;
}
exports.default = xmlQuery;
function query(elements, predicate) {
    switch (typeof predicate) {
        case 'string':
            return xmlQuery(elements
                .map(xmlQuery)
                .find(function (el) { return el.name === predicate; }));
        case 'number':
            return xmlQuery(elements[predicate]);
        default:
            return xmlQuery(elements.map(xmlQuery).find(predicate));
    }
}
function queryAll(elements, predicate) {
    switch (typeof predicate) {
        case 'string':
            return xmlQuery(elements
                .map(xmlQuery)
                .filter(function (el) { return el.name === predicate; }));
        case 'number':
            return xmlQuery(elements[predicate]);
        default:
            return xmlQuery(elements
                .map(xmlQuery)
                .filter(predicate));
    }
}
function forEach(elements, cb, thisArg) {
    return elements
        .map(xmlQuery)
        .forEach(cb, thisArg);
}
function map(elements, cb, thisArg) {
    return xmlQuery(elements
        .map(xmlQuery)
        .map(cb, thisArg));
}
function innerText(elements) {
    return elements
        .filter(function (el) { return el.type === 'text'; })
        .map(function (el) { return el.text; })
        .join('');
}
